/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.dnd.jface;

import static org.fortiss.tooling.kernel.ui.util.DragAndDropUtils.extractDroppedEObject;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.ui.dnd.CompositionServiceLocalTransfer;
import org.fortiss.tooling.kernel.ui.dnd.ElementCompositionDropTargetAdapter;
import org.fortiss.tooling.kernel.ui.util.DragAndDropUtils;
import org.fortiss.tooling.kernel.utils.ModelUtils;

/**
 * {@link ElementCompositionDropTargetAdapter} implementation for a JFace
 * {@link TreeViewer}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class ViewerElementCompositionDropAdapter extends ViewerDropAdapter {

	/** Remember the last target. */
	Object lastTarget = null;

	/** Constructor. */
	public ViewerElementCompositionDropAdapter(Viewer viewer) {
		super(viewer);
		setExpandEnabled(true);
	}

	/** {@inheritDoc} */
	@Override
	public boolean validateDrop(Object target, int operation,
			TransferData transferType) {
		if (target == null) {
			target = getViewer().getInput();
		}
		lastTarget = target;

		if (lastTarget instanceof EObject) {
			EObject dropped = extractDroppedEObject(CompositionServiceLocalTransfer
					.getInstance().getObject());

			return IElementCompositorService.INSTANCE.canCompose(
					(EObject) lastTarget, dropped, null);
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean performDrop(Object data) {
		if (lastTarget instanceof EObject) {
			final EObject target = (EObject) lastTarget;
			final EObject dropped = DragAndDropUtils
					.extractDroppedEObject(data);
			if (IElementCompositorService.INSTANCE.canCompose(target, dropped,
					null)) {
				ModelUtils.runAsCommand(target, new Runnable() {
					@Override
					public void run() {
						IElementCompositorService.INSTANCE.compose(target,
								dropped, null);
					}
				});
				return true;
			}
		}
		return false;
	}

	/** Returns the supported DND operations as defined in {@link DND}. */
	public int getSupportedDNDOperations() {
		return DND.DROP_COPY;
	}

	/** {@inheritDoc} */
	@Override
	public void dragEnter(DropTargetEvent event) {
		event.detail = DND.DROP_COPY;
		super.dragEnter(event);
	}

	/** Returns the preferred transfer for the DND operation. */
	public Transfer[] getPreferredTransfers() {
		return DragAndDropUtils.getCompositionServiceTransfers();
	}
}
